#
#   mtr  --  a network diagnostic tool
#   Copyright (C) 2016  Matt Kimball
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License version 2 as
#   published by the Free Software Foundation.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

'''Infrastructure for running tests which invoke mtr-packet.'''

import fcntl
import os
import select
import socket
import subprocess
import sys
import time
import unittest

#
#  typing is used for mypy type checking, but isn't required to run,
#  so it's okay if we can't import it.
#
try:
    # pylint: disable=locally-disabled, unused-import
    from typing import Dict, List
except ImportError:
    pass


IPV6_TEST_HOST = 'google-public-dns-a.google.com'


class MtrPacketExecuteError(Exception):
    "Exception raised when MtrPacketTest can't execute mtr-packet"
    pass


class ReadReplyTimeout(Exception):
    'Exception raised by TestProbe.read_reply upon timeout'

    pass


class WriteCommandTimeout(Exception):
    'Exception raised by TestProbe.write_command upon timeout'

    pass


class MtrPacketReplyParseError(Exception):
    "Exception raised when MtrPacketReply can't parse the reply string"

    pass


class PacketListenError(Exception):
    'Exception raised when we have unexpected results from mtr-packet-listen'

    pass


def set_nonblocking(file_descriptor):  # type: (int) -> None
    'Put a file descriptor into non-blocking mode'

    flags = fcntl.fcntl(file_descriptor, fcntl.F_GETFL)

    # pylint: disable=locally-disabled, no-member
    fcntl.fcntl(file_descriptor, fcntl.F_SETFL, flags | os.O_NONBLOCK)


def check_for_local_ipv6():
    '''Check for IPv6 support on the test host, to see if we should skip
    the IPv6 tests'''

    addrinfo = socket.getaddrinfo(IPV6_TEST_HOST, 1, socket.AF_INET6)
    if len(addrinfo):
        addr = addrinfo[0][4]

    #  Create a UDP socket and check to see it can be connected to
    #  IPV6_TEST_HOST.  (Connecting UDP requires no packets sent, just
    #  a route present.)
    sock = socket.socket(
        socket.AF_INET6, socket.SOCK_DGRAM, socket.IPPROTO_UDP)

    connect_success = False
    try:
        sock.connect(addr)
        connect_success = True
    except socket.error:
        pass

    sock.close()

    if not connect_success:
        sys.stderr.write(
            'This host has no IPv6.  Skipping IPv6 tests.\n')

    return connect_success


HAVE_IPV6 = check_for_local_ipv6()


# pylint: disable=locally-disabled, too-few-public-methods
class MtrPacketReply(object):
    'A parsed reply from mtr-packet'

    def __init__(self, reply):  # type: (unicode) -> None
        self.token = 0  # type: int
        self.command_name = None  # type: unicode
        self.argument = {}  # type: Dict[unicode, unicode]

        self.parse_reply(reply)

    def parse_reply(self, reply):  # type (unicode) -> None
        'Parses a reply string into members for the instance of this class'

        tokens = reply.split()  # type List[unicode]

        try:
            self.token = int(tokens[0])
            self.command_name = tokens[1]
        except IndexError:
            raise MtrPacketReplyParseError(reply)

        i = 2
        while i < len(tokens):
            try:
                name = tokens[i]
                value = tokens[i + 1]
            except IndexError:
                raise MtrPacketReplyParseError(reply)

            self.argument[name] = value
            i += 2


class PacketListen(object):
    'A test process which listens for a single packet'

    def __init__(self, *args):
        self.process_args = list(args)  # type: List[unicode]
        self.listen_process = None  # type: subprocess.Popen
        self.attrib = None  # type: Dict[unicode, unicode]

    def __enter__(self):
        try:
            self.listen_process = subprocess.Popen(
                ['./mtr-packet-listen'] + self.process_args,
                stdin=subprocess.PIPE,
                stdout=subprocess.PIPE)
        except OSError:
            raise PacketListenError('unable to launch mtr-packet-listen')

        status = self.listen_process.stdout.readline().decode('utf-8')
        if status != 'status listening\n':
            raise PacketListenError('unexpected status')

        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.wait_for_exit()

        self.attrib = {}
        for line in self.listen_process.stdout.readlines():
            tokens = line.decode('utf-8').split()

            if len(tokens) >= 2:
                name = tokens[0]
                value = tokens[1]

                self.attrib[name] = value

        self.listen_process.stdin.close()
        self.listen_process.stdout.close()

    def wait_for_exit(self):
        '''Poll the subprocess for up to ten seconds, until it exits.

        We need to wait for its exit to ensure we are able to read its
        output.'''

        wait_time = 10
        wait_step = 0.1

        steps = int(wait_time / wait_step)

        exit_value = None

        # pylint: disable=locally-disabled, unused-variable
        for i in range(steps):
            exit_value = self.listen_process.poll()
            if exit_value is not None:
                break

            time.sleep(wait_step)

        if exit_value is None:
            raise PacketListenError('mtr-packet-listen timeout')

        if exit_value != 0:
            raise PacketListenError('mtr-packet-listen unexpected error')


class MtrPacketTest(unittest.TestCase):
    '''Base class for tests invoking mtr-packet.

    Start a new mtr-packet subprocess for each test, and kill it
    at the conclusion of the test.

    Provide methods for writing commands and reading replies.
    '''

    def __init__(self, *args):
        self.reply_buffer = None  # type: unicode
        self.packet_process = None  # type: subprocess.Popen
        self.stdout_fd = None  # type: int

        super(MtrPacketTest, self).__init__(*args)

    def setUp(self):
        'Set up a test case by spawning a mtr-packet process'

        packet_path = os.environ.get('MTR_PACKET', './mtr-packet')

        self.reply_buffer = ''
        try:
            self.packet_process = subprocess.Popen(
                [packet_path],
                stdin=subprocess.PIPE,
                stdout=subprocess.PIPE)
        except OSError:
            raise MtrPacketExecuteError(packet_path)

        #  Put the mtr-packet process's stdout in non-blocking mode
        #  so that we can read from it without a timeout when
        #  no reply is available.
        self.stdout_fd = self.packet_process.stdout.fileno()
        set_nonblocking(self.stdout_fd)

        self.stdin_fd = self.packet_process.stdin.fileno()
        set_nonblocking(self.stdin_fd)

    def tearDown(self):
        'After a test, kill the running mtr-packet instance'

        self.packet_process.stdin.close()
        self.packet_process.stdout.close()

        try:
            self.packet_process.kill()
        except OSError:
            return

    def parse_reply(self, timeout=10.0):  # type: (float) -> MtrPacketReply
        '''Read the next reply from mtr-packet and parse it into
        an MtrPacketReply object.'''

        reply_str = self.read_reply(timeout)

        return MtrPacketReply(reply_str)

    def read_reply(self, timeout=10.0):  # type: (float) -> unicode
        '''Read the next reply from mtr-packet.

        Attempt to read the next command reply from mtr-packet.  If no reply
        is available withing the timeout time, raise ReadReplyTimeout
        instead.'''

        start_time = time.time()

        #  Read from mtr-packet until either the timeout time has elapsed
        #  or we read a newline character, which indicates a finished
        #  reply.
        while True:
            now = time.time()
            elapsed = now - start_time

            select_time = timeout - elapsed
            if select_time < 0:
                select_time = 0

            select.select([self.stdout_fd], [], [], select_time)

            reply_bytes = None

            try:
                reply_bytes = os.read(self.stdout_fd, 1024)
            except OSError:
                pass

            if reply_bytes:
                self.reply_buffer += reply_bytes.decode('utf-8')

            #  If we have read a newline character, we can stop waiting
            #  for more input.
            newline_ix = self.reply_buffer.find('\n')
            if newline_ix != -1:
                break

            if elapsed >= timeout:
                raise ReadReplyTimeout()

        reply = self.reply_buffer[:newline_ix]
        self.reply_buffer = self.reply_buffer[newline_ix + 1:]
        return reply

    def write_command(self, cmd, timeout=10.0):
        # type: (unicode, float) -> None

        '''Send a command string to the mtr-packet instance, timing out
        if we are unable to write for an extended period of time.  The
        timeout is to avoid deadlocks with the child process where both
        the parent and the child are writing to their end of the pipe
        and expecting the other end to be reading.'''

        command_str = cmd + '\n'
        command_bytes = command_str.encode('utf-8')

        start_time = time.time()

        while True:
            now = time.time()
            elapsed = now - start_time

            select_time = timeout - elapsed
            if select_time < 0:
                select_time = 0

            select.select([], [self.stdin_fd], [], select_time)

            bytes_written = 0
            try:
                bytes_written = os.write(self.stdin_fd, command_bytes)
            except OSError:
                pass

            command_bytes = command_bytes[bytes_written:]
            if not len(command_bytes):
                break

            if elapsed >= timeout:
                raise WriteCommandTimeout()


def check_running_as_root():
    'Print a warning to stderr if we are not running as root.'

    # pylint: disable=locally-disabled, no-member
    if sys.platform != 'cygwin' and os.getuid() > 0:
        sys.stderr.write(
            'Warning: many tests require running as root\n')



#!/usr/bin/env python
#
#   mtr  --  a network diagnostic tool
#   Copyright (C) 2016  Matt Kimball
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License version 2 as
#   published by the Free Software Foundation.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

'''Test sending probes and receiving respones.'''

import socket
import sys
import time
import unittest

import mtrpacket


def resolve_ipv6_address(hostname):  # type: (str) -> str
    'Resolve a hostname to an IP version 6 address'

    for addrinfo in socket.getaddrinfo(hostname, 0):
        # pylint: disable=locally-disabled, unused-variable
        (family, socktype, proto, name, sockaddr) = addrinfo

        if family == socket.AF_INET6:
            sockaddr6 = sockaddr  # type: tuple

            (address, port, flow, scope) = sockaddr6
            return address

    raise LookupError(hostname)


def check_feature(test, feature):
    'Check for support for a particular feature with mtr-packet'

    check_cmd = '70 check-support feature ' + feature
    test.write_command(check_cmd)

    reply = test.parse_reply()
    test.assertEqual(reply.command_name, 'feature-support')
    test.assertIn('support', reply.argument)

    if reply.argument['support'] != 'ok':
        return False

    return True


def test_basic_remote_probe(test, ip_version, protocol):
    'Test a probe to a remote host with a TTL of 1'

    protocol_str = 'protocol ' + protocol
    if ip_version == 6:
        address_str = 'ip-6 ' + resolve_ipv6_address(mtrpacket.IPV6_TEST_HOST)
    elif ip_version == 4:
        address_str = 'ip-4 8.8.8.8'
    else:
        raise ValueError(ip_version)

    cmd = '60 send-probe ' + \
        protocol_str + ' ' + address_str + ' port 164 ttl 1'
    test.write_command(cmd)

    reply = test.parse_reply()
    test.assertEqual(reply.command_name, 'ttl-expired')


def test_basic_local_probe(test, ip_version, protocol):
    'Test a probe to a closed port on localhost'

    protocol_str = 'protocol ' + protocol
    if ip_version == 6:
        address_str = 'ip-6 ::1'
    elif ip_version == 4:
        address_str = 'ip-4 127.0.0.1'

    cmd = '61 send-probe ' + \
        protocol_str + ' ' + address_str + ' port 164'
    test.write_command(cmd)

    reply = test.parse_reply()
    test.assertEqual(reply.command_name, 'reply')

    if ip_version == 6:
        test.assertIn('ip-6', reply.argument)
        test.assertEqual(reply.argument['ip-6'], '::1')
    elif ip_version == 4:
        test.assertIn('ip-4', reply.argument)
        test.assertEqual(reply.argument['ip-4'], '127.0.0.1')


def test_basic_probe(test, ip_version, protocol):
    # type: (mtrpacket.MtrPacketTest, int, unicode) -> None

    '''Test a probe with TTL expiration and a probe which reaches its
    destination with a particular protocol.'''

    if not check_feature(test, protocol):
        err_str = 'Skipping ' + protocol + ' test due to no support\n'
        sys.stderr.write(err_str.encode('utf-8'))
        return

    test_basic_remote_probe(test, ip_version, protocol)
    test_basic_local_probe(test, ip_version, protocol)


class TestProbeICMPv4(mtrpacket.MtrPacketTest):
    '''Test sending probes using IP version 4'''

    def test_probe(self):
        'Test sending regular ICMP probes to known addresses'

        #  Probe Google's well-known DNS server and expect a reply
        self.write_command('14 send-probe ip-4 8.8.8.8')
        reply = self.parse_reply()
        self.assertEqual(reply.token, 14)
        self.assertEqual(reply.command_name, 'reply')
        self.assertIn('ip-4', reply.argument)
        self.assertEqual(reply.argument['ip-4'], '8.8.8.8')
        self.assertIn('round-trip-time', reply.argument)

    def test_timeout(self):
        'Test timeouts when sending to a non-existant address'

        #
        #  Probe a non-existant address, and expect no reply
        #
        #  I'm not sure what the best way to find an address that doesn't
        #  exist, but is still route-able.  If we use a reserved IP
        #  address range, Windows will tell us it is non-routeable,
        #  rather than timing out when transmitting to that address.
        #
        #  We're just using a currently unused address in Google's
        #  range instead.  This is probably not the best solution.
        #

        # pylint: disable=locally-disabled, unused-variable
        for i in range(16):
            self.write_command('15 send-probe ip-4 8.8.254.254 timeout 1')
            reply = self.parse_reply()
            self.assertEqual(reply.token, 15)
            self.assertEqual(reply.command_name, 'no-reply')

    def test_exhaust_probes(self):
        'Test exhausting all available probes'

        probe_count = 4 * 1024
        token = 1024

        # pylint: disable=locally-disabled, unused-variable
        for i in range(probe_count):
            command = str(token) + ' send-probe ip-4 8.8.254.254 timeout 60'
            token += 1
            self.write_command(command)

            reply = None
            try:
                reply = self.parse_reply(0)
            except mtrpacket.ReadReplyTimeout:
                pass

            if reply:
                if reply.command_name == 'probes-exhausted':
                    break

        self.assertIsNotNone(reply)
        self.assertEqual(reply.command_name, 'probes-exhausted')

    def test_timeout_values(self):
        '''Test that timeout values wait the right amount of time

        Give each probe a half-second grace period to probe a timeout
        reply after the expected timeout time.'''

        begin = time.time()
        self.write_command('19 send-probe ip-4 8.8.254.254 timeout 0')
        self.parse_reply()
        elapsed = time.time() - begin
        self.assertLess(elapsed, 0.5)

        begin = time.time()
        self.write_command('20 send-probe ip-4 8.8.254.254 timeout 1')
        self.parse_reply()
        elapsed = time.time() - begin
        self.assertGreaterEqual(elapsed, 0.9)
        self.assertLess(elapsed, 1.5)

        begin = time.time()
        self.write_command('21 send-probe ip-4 8.8.254.254 timeout 3')
        self.parse_reply()
        elapsed = time.time() - begin
        self.assertGreaterEqual(elapsed, 2.9)
        self.assertLess(elapsed, 3.5)

    def test_ttl_expired(self):
        'Test sending a probe which will have its time-to-live expire'

        #  Probe Goolge's DNS server, but give the probe only one hop
        #  to live.
        self.write_command('16 send-probe ip-4 8.8.8.8 ttl 1')
        reply = self.parse_reply()
        self.assertEqual(reply.command_name, 'ttl-expired')
        self.assertIn('ip-4', reply.argument)
        self.assertIn('round-trip-time', reply.argument)

    def test_parallel_probes(self):
        '''Test sending multiple probes in parallel

        We will expect the probes to complete out-of-order by sending
        a probe to a distant host immeidately followed by a probe to
        the local host.'''

        success_count = 0
        loop_count = 32

        # pylint: disable=locally-disabled, unused-variable
        for i in range(loop_count):
            #  Probe the distant host before the local host.
            self.write_command('17 send-probe ip-4 8.8.8.8 timeout 1')
            self.write_command('18 send-probe ip-4 127.0.0.1 timeout 1')

            reply = self.parse_reply()
            if reply.command_name == 'no-reply':
                continue

            self.assertEqual(reply.command_name, 'reply')
            self.assertIn('ip-4', reply.argument)
            self.assertEqual(reply.argument['ip-4'], '127.0.0.1')
            self.assertIn('round-trip-time', reply.argument)
            first_time = int(reply.argument['round-trip-time'])

            reply = self.parse_reply()
            if reply.command_name == 'no-reply':
                continue

            self.assertEqual(reply.command_name, 'reply')
            self.assertIn('ip-4', reply.argument)
            self.assertEqual(reply.argument['ip-4'], '8.8.8.8')
            self.assertIn('round-trip-time', reply.argument)
            second_time = int(reply.argument['round-trip-time'])

            #  Ensure we got a reply from the host with the lowest latency
            #  first.
            self.assertLess(first_time, second_time)

            success_count += 1

        #  We need 90% success to pass.  This allows a few probes to be
        #  occasionally dropped by the network without failing the test.
        required_success = int(loop_count * 0.90)
        self.assertGreaterEqual(success_count, required_success)


class TestProbeICMPv6(mtrpacket.MtrPacketTest):
    '''Test sending probes using IP version 6'''

    def __init__(self, *args):
        google_addr = resolve_ipv6_address(mtrpacket.IPV6_TEST_HOST)

        self.google_addr = google_addr  # type: str

        super(TestProbeICMPv6, self).__init__(*args)

    @unittest.skipUnless(mtrpacket.HAVE_IPV6, 'No IPv6')
    def test_probe(self):
        "Test a probe to Google's public DNS server"

        #  Probe Google's well-known DNS server and expect a reply
        self.write_command('51 send-probe ip-6 ' + self.google_addr)
        reply = self.parse_reply()
        self.assertEqual(reply.command_name, 'reply')
        self.assertIn('ip-6', reply.argument)
        self.assertIn('round-trip-time', reply.argument)

        #  Probe the loopback, and check the address we get a reply from is
        #  also the loopback.  While implementing IPv6, I had a bug where
        #  the low bits of the received address got zeroed.  This checks for
        #  that bug.
        self.write_command('52 send-probe ip-6 ::1')
        reply = self.parse_reply()
        self.assertEqual(reply.command_name, 'reply')
        self.assertIn('ip-6', reply.argument)
        self.assertIn('round-trip-time', reply.argument)
        self.assertEqual(reply.argument['ip-6'], '::1')

    @unittest.skipUnless(mtrpacket.HAVE_IPV6, 'No IPv6')
    def test_ttl_expired(self):
        'Test sending a probe which will have its time-to-live expire'

        #  Probe Goolge's DNS server, but give the probe only one hop
        #  to live.
        cmd = '53 send-probe ip-6 ' + self.google_addr + ' ttl 1'
        self.write_command(cmd)
        reply = self.parse_reply()
        self.assertEqual('ttl-expired', reply.command_name)
        self.assertIn('ip-6', reply.argument)
        self.assertIn('round-trip-time', reply.argument)


class TestProbeUDP(mtrpacket.MtrPacketTest):
    'Test transmitting probes using UDP'

    def udp_port_test(self, address):  # type: (unicode) -> None
        'Test UDP probes with variations on source port and dest port'

        if not check_feature(self, 'udp'):
            return

        cmd = '80 send-probe protocol udp ' + address
        self.write_command(cmd)
        reply = self.parse_reply()
        self.assertEqual('reply', reply.command_name)

        cmd = '81 send-probe protocol udp port 990 ' + address
        self.write_command(cmd)
        reply = self.parse_reply()
        self.assertEqual('reply', reply.command_name)

        cmd = '82 send-probe protocol udp local-port 1991 ' + address
        self.write_command(cmd)
        reply = self.parse_reply()
        self.assertEqual('reply', reply.command_name)

    def test_udp_v4(self):
        'Test IPv4 UDP probes'

        test_basic_probe(self, 4, 'udp')

        self.udp_port_test('ip-4 127.0.0.1')

    @unittest.skipUnless(mtrpacket.HAVE_IPV6, 'No IPv6')
    def test_udp_v6(self):
        'Test IPv6 UDP probes'

        test_basic_probe(self, 6, 'udp')

        self.udp_port_test('ip-6 ::1')


class TestProbeTCP(mtrpacket.MtrPacketTest):
    'Test TCP probe support'

    def test_tcp_v4(self):
        '''Test IPv4 TCP probes, with TTL expiration, to a refused port
        and to an open port'''

        test_basic_probe(self, 4, 'tcp')

        if not check_feature(self, 'tcp'):
            return

        #  Probe a local port assumed to be open  (ssh)
        cmd = '80 send-probe ip-4 127.0.0.1 protocol tcp port 22'
        self.write_command(cmd)

        reply = self.parse_reply()
        self.assertEqual(reply.command_name, 'reply')

    @unittest.skipUnless(mtrpacket.HAVE_IPV6, 'No IPv6')
    def test_tcp_v6(self):
        'Test IPv6 TCP probes'

        test_basic_probe(self, 6, 'tcp')

        if not check_feature(self, 'tcp'):
            return

        #  Probe a local port assumed to be open  (ssh)
        cmd = '80 send-probe ip-6 ::1 protocol tcp port 22'
        self.write_command(cmd)

        reply = self.parse_reply()
        self.assertEqual(reply.command_name, 'reply')


class TestProbeSCTP(mtrpacket.MtrPacketTest):
    'Test SCTP probes'

    def test_sctp_v4(self):
        'Test basic SCTP probes over IPv4'

        test_basic_probe(self, 4, 'sctp')

    @unittest.skipUnless(mtrpacket.HAVE_IPV6, 'No IPv6')
    def test_sctp_v6(self):
        'Test basic SCTP probes over IPv6'

        test_basic_probe(self, 6, 'sctp')


if __name__ == '__main__':
    mtrpacket.check_running_as_root()
    unittest.main()



#!/usr/bin/env python
#
#   mtr  --  a network diagnostic tool
#   Copyright (C) 2016  Matt Kimball
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License version 2 as
#   published by the Free Software Foundation.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

'''Test probe customization parameters'''

import sys
import unittest

import mtrpacket


@unittest.skipIf(sys.platform == 'cygwin', 'No Cygwin test')
class TestParameters(mtrpacket.MtrPacketTest):
    'Use parameter arguments to mtr-packet and examine the resulting packet'

    def test_size(self):
        'Test probes sent with an explicit packet size'

        with mtrpacket.PacketListen('-4') as listen:
            cmd = '20 send-probe ip-4 127.0.0.1 size 512'

            self.write_command(cmd)

        self.assertEqual(listen.attrib['size'], '512')

    def test_pattern(self):
        'Test probes are filled with the requested bit pattern'

        with mtrpacket.PacketListen('-4') as listen:
            cmd = '20 send-probe ip-4 127.0.0.1 bit-pattern 44'

            self.write_command(cmd)

        self.assertEqual(listen.attrib['bitpattern'], '44')

    def test_tos(self):
        'Test setting the TOS field'

        with mtrpacket.PacketListen('-4') as listen:
            cmd = '20 send-probe ip-4 127.0.0.1 tos 62'

            self.write_command(cmd)

        self.assertEqual(listen.attrib['tos'], '62')


@unittest.skipIf(sys.platform == 'cygwin', 'No Cygwin test')
class TestIPv6Parameters(mtrpacket.MtrPacketTest):
    'Test packet paramter customization for IPv6'

    @unittest.skipUnless(mtrpacket.HAVE_IPV6, 'No IPv6')
    def test_param(self):
        'Test a variety of packet parameters'

        with mtrpacket.PacketListen('-6') as listen:
            param = 'size 256 bit-pattern 51 tos 77'
            cmd = '20 send-probe ip-6 ::1 ' + param

            self.write_command(cmd)

        self.assertEqual(listen.attrib['size'], '256')
        self.assertEqual(listen.attrib['bitpattern'], '51')


if __name__ == '__main__':
    mtrpacket.check_running_as_root()
    unittest.main()



#!/usr/bin/env python
#
#   mtr  --  a network diagnostic tool
#   Copyright (C) 2016  Matt Kimball
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License version 2 as
#   published by the Free Software Foundation.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

'''Test mtr-packet's command parsing.'''


import time
import unittest

import mtrpacket


class TestCommandParse(mtrpacket.MtrPacketTest):
    '''Test cases with malformed commands and version checks'''

    def test_unknown_command(self):
        'Test sending a command unknown to mtr-packet'

        self.write_command('13 argle-bargle')
        self.assertEqual(self.read_reply(), '13 unknown-command')

    def test_malformed_command(self):
        'Test sending a malformed command request to mtr-packet'

        self.write_command('malformed')
        self.assertEqual(self.read_reply(), '0 command-parse-error')

    def test_exit_on_stdin_closed(self):
        '''Test that the packet process terminates after stdin is closed

        Test that, when outstanding requests are complete, the process
        terminates following stdin being closed.'''

        self.write_command('15 send-probe ip-4 8.8.254.254 timeout 1')
        self.packet_process.stdin.close()
        time.sleep(2)
        self.read_reply()
        exit_code = self.packet_process.poll()
        self.assertIsNotNone(exit_code)

    def test_invalid_argument(self):
        'Test sending invalid arguments with probe requests'

        bad_commands = [
            '22 send-probe',
            '23 send-probe ip-4 str-value',
            '24 send-probe ip-4 8.8.8.8 timeout str-value',
            '25 send-probe ip-4 8.8.8.8 ttl str-value',
        ]

        for cmd in bad_commands:
            self.write_command(cmd)
            reply = self.parse_reply()
            self.assertEqual(reply.command_name, 'invalid-argument')

    def test_versioning(self):
        'Test version checks and feature support checks'

        feature_tests = [
            ('31 check-support feature ip-4', 'ok'),
            ('32 check-support feature send-probe', 'ok'),
            ('33 check-support feature bogus-feature', 'no')
        ]

        self.write_command('30 check-support feature version')
        reply = self.parse_reply()
        self.assertEqual(reply.token, 30)
        self.assertEqual(reply.command_name, 'feature-support')
        self.assertIn('support', reply.argument)

        for (request, expected) in feature_tests:
            self.write_command(request)
            reply = self.parse_reply()
            self.assertEqual(reply.command_name, 'feature-support')
            self.assertIn('support', reply.argument)
            self.assertEqual(reply.argument['support'], expected)

    def test_command_overflow(self):
        'Test overflowing the incoming command buffer'

        big_buffer = 'x' * (64 * 1024)
        self.write_command(big_buffer)

        reply = self.read_reply()
        self.assertEqual(reply, '0 command-buffer-overflow')


if __name__ == '__main__':
    mtrpacket.check_running_as_root()
    unittest.main()
